{Modifications of original GeoGet WPT code by pmoravec} 
{Version 0.9.7}
{Changes since 0.8 - Elevation support, Bugfix for unchecked offline information} 
{Changes since 0.9.1 - Suppressed waypoints for found caches + checkbox setting - DrakMrak}
{Changes since 0.9.2 - Show setting window}
{Changes since 0.9.3 - Auto Install version}
{Changes since 0.9.4 - Waymarking waypoints}
{Changes since 0.9.5 - Fixed multi-line description in additional waypoints}
{Changes since 0.9.6 - Fixed invalid classification of some caches as waymarks}
{Changes since 0.9.7 - Fixed invalid length sum of name and hint }
{Feel free to use or modify}

//Geoget 2

const
  defColorize     = True;
  defShowHint     = True;
  defReverseHint  = False;
  defCropHint     = False;
  defCropLength   = 300;
  defOffline      = True;
  defOZICE1       = False;
  defCustomURL    = '\Storage Card\offline';
  defProximity    = 0;
  tagelev         = 'Elevation'; {From Elevation import script}
  defFoundWptOn   = True;	 //Show waypoints
  defPrompt       = True;    //Show setting

var
  counter: integer;
  colorize: boolean;
  showHint: boolean;
  revHint: boolean;
  cropHint: boolean;
  cropLength: integer;
  useOffline: boolean;
  forOZICE1: boolean;
  customURL: string;
  proximity: integer;
  foundWptOn: boolean;		//Show or Hidde waypoints (no cache - only waypoints)
  prompt: boolean;			//Show or Hidee setting windows

procedure LoadSettings;
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(Geoget_datadir + '\wpt-adv-gg2.ini');
  try
    colorize   := iniF.readBool('General', 'Colorize', defColorize);
    showHint   := iniF.readBool('General', 'ShowHint', defShowHint);
    proximity  := iniF.readInteger('General', 'WaypointProximity', defProximity);
    revHint    := iniF.readBool('Hint', 'ReverseHint', defReverseHint);
    cropHint   := iniF.readBool('Hint', 'CropHint', defCropHint);
    cropLength := iniF.readInteger('Hint', 'CropLength', defCropLength);
    useOffline := iniF.readBool('Offline', 'Generate', defOffline);
    forOZICE1  := iniF.readBool('Offline', 'OziCE1', defOZICE1);
    customURL  := iniF.readString('Offline', 'CustomURL', defCustomURL);
	  foundWptOn := iniF.readBool('General', 'ShowWptAtFoundedCache', defFoundWptOn);
	  prompt     := iniF.readBool('General', 'ShowSetting', defPrompt);
  finally
    iniF.Free;
  end;
end;

const
  border  = 10; 
  baseh   = 30; // Lineskip
  basew   = 400; // Max item Width

function InitSettings : boolean;
var
  Form: TForm;
  URL: TEdit;
  Color: TCheckBox;
  Hint: TCheckBox;
  Reverse: TCheckBox;
  Crop: TCheckBox;
  Offline : TCheckBox;
  PDA : TCheckBox;
  WptOn : TCheckBox;
  SetPrompt : TCheckBox;
  Button: TButton;
  Button2: TButton;
  Button3: TButton;
  ProxLabel: TLabel;
  Comm1:TLabel;
  Comm2:Tlabel;
  ProxInp: TEdit;
  CropInp: TEdit;
  frmresult : Integer;
  tpos : Integer;
  
begin
  if prompt then
    begin
	  tpos := border;

	  Form := TForm.Create(nil);
	  Form.Width := basew + 2*border;
	  Form.Height := 420; {Pre-defined, will change in the end}

	  Form.BorderStyle := bsDialog;
	  Form.BorderIcons := [];
	  Form.Caption := 'Set WPT Export Options';
	  Form.Position := poScreenCenter;
	  
	  Color := TCheckBox.Create(Form);
	  Color.Caption := 'Waypoint &colors';
	  Color.Checked := colorize;
	  Color.Parent  := Form;
	  Color.SetBounds(border,tpos,basew,24);
	  tpos := tpos + baseh;

	  WptOn := TCheckBox.Create(Form);
	  WptOn.Caption := 'Show Waypoints from &Found Caches';
	  WptOn.Checked := foundWptOn;
	  WptOn.Parent  := Form;
	  WptOn.SetBounds(border, tpos, basew,24);
	  tpos := tpos + baseh;
	  
	  Hint := TCheckBox.Create(Form);
	  Hint.Caption := 'Add &Hints to Waypoint Descriptions';
	  Hint.Checked := showHint;
	  Hint.Parent  := Form;
	  Hint.SetBounds(border,tpos,basew,24);
	  tpos := tpos + baseh;

	  Reverse := TCheckBox.Create(Form);
	  Reverse.Caption := 'Spell the Hint Backwa&rds';
	  Reverse.Checked := revHint;
	  Reverse.Parent  := Form;
	  Reverse.SetBounds(4 * border,tpos,basew-4*border,24);
	  tpos := tpos + baseh;

	  Crop := TCheckBox.Create(Form);
	  Crop.Caption := '&Crop Hints and WP Descriptions. Max. Length [chars]:';
	  Crop.Checked := cropHint;
	  Crop.Parent  := Form;
	  Crop.SetBounds(border,tpos,basew-40-2*border,24);

	  CropInp := TEdit.Create(Form);
	  CropInp.Font.Name := 'Tahoma';
	  CropInp.SetBounds(border+Crop.Width+Crop.Left,tpos,40,24);
	  CropInp.Parent := Form;
	  CropInp.Text := intToStr(cropLength);
	  tpos := tpos + baseh;

	  Offline := TCheckBox.Create(Form);
	  Offline.Checked := useOffline;
	  Offline.SetBounds(border,tpos,basew,24);
	  Offline.Caption := 'Add Reference to &Offline files copied to:';
	  Offline.Parent := Form;
	  tpos := tpos + baseh;

	  URL := TEdit.Create(Form);
	  URL.Font.Name := 'Tahoma';
	  URL.SetBounds(4*border,tpos,basew-4*border,24);
	  URL.Parent := Form;
	  URL.Text := customURL;
	  tpos := tpos + baseh;

	  PDA := TCheckBox.Create(Form);
	  PDA.Checked := forOZICE1;
	  PDA.SetBounds(4*border,tpos,basew-4*border,24);
	  PDA.Caption := 'OziCE &1.x Export (to "Data File Path" subdirectory ---^) ';
	  PDA.Parent := Form;
	  tpos := tpos + baseh;

	  ProxLabel := TLabel.Create(Form);
	  ProxLabel.SetBounds(border,tpos,100,24);
	  ProxLabel.Caption := 'Waypoint proximity [m] (0 to disable):';
	  ProxLabel.Parent := Form;

	  ProxInp := TEdit.Create(Form);
	  ProxInp.SetBounds(border+ProxLabel.Width+ProxLabel.Left,tpos-4,40,24);
	  ProxInp.Parent := Form;
	  ProxInp.Font.Name := 'Tahoma';
	  ProxInp.Text := IntToStr(proximity);
	  tpos := tpos + baseh;

	  Button := TButton.Create(Form);
	  Button.Left := border + 20;
	  Button.Top := tpos;
	  Button.Width := 80;
	  Button.Height := 24;
	  Button.Caption := '&Apply';
	  Button.Parent := Form;
	  Button.Default := True;
	  Button.Modalresult := mrOk;

	  Button2 := TButton.Create(Form);
	  Button2.Left := Button.Width + Button.Left + border;
	  Button2.Top := tpos;
	  Button2.Width := 110;
	  Button2.Height := 24;
	  Button2.Caption := 'Apply + &Save';
	  Button2.Parent := Form;
	  Button2.Modalresult := mrYes;

	  Button3 := TButton.Create(Form);
	  Button3.Left := Button2.Width + Button2.Left + border;
	  Button3.Top := tpos;
	  Button3.Width := 130;
	  Button3.Height := 24;
	  Button3.Caption := 'Use O&ld Settings';
	  Button3.Cancel := True;
	  Button3.Parent := Form;
	  Button3.Modalresult := mrCancel;
	  tpos := tpos + baseh;
	  
	  SetPrompt := TCheckBox.Create(Form);
	  SetPrompt.Caption := 'Show Setting';
	  SetPrompt.Checked := prompt;
	  SetPrompt.Parent  := Form;
	  SetPrompt.SetBounds(border, tpos, basew,24);
	  tpos := tpos + 20;
	  
	  Comm1 := TLabel.Create(Form);
	  Comm1.Font.Name := 'Arial Narrow';
	  Comm1.SetBounds(border + 20,tpos,basew,24);
	  Comm1.Caption := '(if save unchecked you have to manually set the value of "ShowSetting = 1"';
	  Comm1.Parent := Form;
	  tpos := tpos + 16;
	  
	  Comm2 := TLabel.Create(Form);
	  Comm2.Font.Name := 'Arial Narrow';
	  Comm2.SetBounds(border + 20,tpos,basew,24);
	  Comm2.Caption := 'in the appropriate INI file, if you want to see settings again)';
	  Comm2.Parent := Form;
	  tpos := tpos + 16;
	  
	  Form.Height := tpos + baseh; 

	  frmresult := form.showmodal;
  
	  if frmresult =  mrYes  then
		Result := True
	  else
	   Result:= False;
	   
	  if (frmresult = mrYes) or (frmresult = mrOk) then
		begin
		  customURL := URL.text;
		  colorize  := Color.checked;
		  foundWptOn:= WptOn.checked;
		  showHint  := Hint.checked; 
		  revHint   := Reverse.checked; 
		  cropHint  := Crop.checked; 
		  useOffline:= Offline.checked;
		  forOZICE1 := PDA.checked;
		  proximity := StrToIntDef(ProxInp.text,-1); 
		  cropLength := StrToIntDef(CropInp.Text,0);
		  prompt := SetPrompt.checked;
		end
	  else
		LoadSettings;
    end
  else
    LoadSettings;
  //-----------------------------------------------------------
  if useOffline and ((forOZICE1) xor ((customURL[1] <> '/') and (customURL[1] <> '\'))) then
  begin
     if (forOZICE1) then 
       ShowMessage('You must use relative path (starting at OZI CE "Data file path" set in "Map and Data Files") for OZI_CE 1.x!' + CRLF + 'Please correct your offline data directory now (most likely to ''offline'') and click on ''Apply && Save''.')
     else
       ShowMessage('You must use absolute path starting with \ in OziCE 2.x or C:\ in OZIExplorer!' + CRLF + 'Please correct your offline data directory now (most likely to ''\Storage Card\offline'') and click on ''Apply && Save''.');
     Result := InitSettings;
  end;
  if (proximity < 0) then
  begin
      ShowMessage('Invalid Proximity value ''' + ProxInp.text + '''!' + CRLF
            +'Please correct it to non-negative value (without decimal places).');
      proximity := defProximity;
     Result := InitSettings;
  end;
  if (cropLength <= 0) then
  begin
      ShowMessage('Invalid Crop value ''' + CropInp.text + '''!' + CRLF
            +'Please correct it to possitive value (without decimal places).');
      cropLength := defCropLength;
     Result := InitSettings;
  end;
end;


procedure SaveSettings;
var
  iniF: TIniFile;
begin
  iniF := TIniFile.Create(Geoget_datadir + '\wpt-adv-gg2.ini');
  try
    iniF.writeBool('General', 'Colorize', colorize);
    iniF.writeBool('General', 'ShowHint', showHint);
    iniF.writeBool('Hint', 'ReverseHint', revHint);
    iniF.writeBool('Hint', 'CropHint', cropHint);
    iniF.writeInteger('Hint', 'CropLength', cropLength);
    iniF.writeBool('Offline', 'Generate', useOffline);
    iniF.writeBool('Offline', 'OziCE1', forOZICE1);
    iniF.writeString('Offline', 'CustomURL', customURL );
    iniF.writeInteger('General', 'WaypointProximity', proximity);
	iniF.writeBool('General', 'ShowWptAtFoundedCache', foundWptOn);
	iniF.writeBool('General', 'ShowSetting', prompt);
  finally
    iniF.Free;
  end;
end;


function ExportExtension: string;
begin
  result := 'WPT';
end;

function ExportDescription: string;
begin
  result := 'OziExp WPT File with Advanced Options';
end;

function ExportHeader: string;
begin
  LoadSettings;
  if InitSettings then SaveSettings;
  Result := 'OziExplorer Waypoint File Version 1.2' + CRLF;
  Result := Result + 'WGS 84' + CRLF;
  Result := Result + 'Reserved 2' + CRLF;
  Result := Result + 'Reserved 3' + CRLF;
  counter := 1;
end;

function ExportFooter: string;
begin
  if forOZICE1 and (counter > 1000) then
    MessBox(inttostr(counter) + ' points were exported, but only 1000 of them will be accessible in OZICE 1.x! Try to reduce the number of points in your selection.', 'Warning', 0);
  result := '';
end;

function ColorPoint: string;
var
  color: string;
  bgcolor: string;
begin
  if colorize then 
  begin
	  color := '0'; 			    		{Default text is Black}
	  bgcolor := '65535';                   {Unlisted type - Black on Yellow}
	  if Pos('WM',GC.ID) <> 1 then begin
	    if length(GC.IDTag) = 4 then		{Is the cache code present?}
	    case Ord(GC.IDTag[1]) of		    {--if so, use the first letter value}
		 69 : bgcolor := '9869006';         {Event - pink from Event icon} 
		 71 : bgcolor := '14734496';        {EarthCache - Aqua} 
		 72 : bgcolor := '16575206';        {LetterBox Hybrid - Envelope icon color}
		 73 : bgcolor := '9441424';         {WhereIGo - Blue from WIG icon}
		 77 : begin
			 color   := '16777215';         {Multi} 
			 bgcolor := '827634';           {White on Average of orange box colors}
		      end;
		 84 : begin
			 color   := '16777215';         {Traditional}
			 bgcolor := '3901272';          {White on avg. of Green box colors}
		      end;
		 85 : begin
			 color   := '16777215';
			 bgcolor := '12657920';         {Unknown - White on Dark Blue}
		      end;
		 86 : bgcolor := '16575206';        {Virtual - 1/3 Gray}
		 87 : bgcolor := '7500402';         {Webcam - 55% Gray}	 
	    end;
      end		
	  else begin
	    color := '16777215';				{WM - White}
	    bgcolor := '128';	     			{WM - Maroon}
	  end;
	  
	  if GC.IsOwner then
	  begin
	    {color := bgcolor;}
	    bgcolor := '15631086'; 
	  end;

	  if GC.IsFound then                 {Found}
	  begin
	    color := bgcolor;
	    bgcolor := '12382140';	         {Cache color on Green (from cache list)}
	  end;

	  if Pos('WM',GC.ID) <> 1 then begin
	    if GC.IsArchived then            {Archived}
	    begin
	      color := bgcolor;		
	      bgcolor := '255';	             {Cache Color on Light Red Background}
	    end
	    else if GC.IsDisabled then
	    begin
	      color := bgcolor;
	      bgcolor := '3355647';	         {Cache Color on Red Background}
	    end;
	  end;
      Result := color + ',' + bgcolor;
  end
  else
    Result := '0,65535';
end;

function ColorWaypoint(wpt: TWpt): string;
var
  color: string;
  bgcolor: string;
begin
  color     := '0';                      {Black text by default}
  bgcolor   := '65535';                  {Yellow background}

  if colorize then
  begin
	  if wpt.IsFinal then
	  begin
	    color   := '16777215';
	    bgcolor := '57504';
	  end
	  else if wpt.IsUserWaypoint then
	  begin
	    color   := '8388608';              {Blue on Yellow Background}
	    bgcolor := '65535';
	  end	    
	  else if GC.IsOwner then
	  begin
	    {color := bgcolor;}
	    bgcolor := '15646958'; 
	  end;
	 
	  if GC.IsFound then
	  begin
	    color   := '32768';                {Green on Yellow background}
	    bgcolor := '10092543';          {Yellow Background (from cache list)}
	  end	    
	  else if GC.IsDisabled or GC.IsArchived then
	  begin
	    color := bgcolor;
	    bgcolor := '5263615';	     {Cache Color on Light Red Background}
	  end;
  end;
  Result := color + ',' + bgcolor;
end;

function m2feet(m: string) : string;
var
  height : extended;
begin
  height := strtofl(m) / 0.3048;
  Result  := inttostr(round(height));
end;

function FixDesc(desc: string; cropHint: boolean; cropLength: integer): string;
begin
   desc := ReplaceString(
             ReplaceString(ReplaceString(ReplaceString(UtfToAscii(desc), Chr($0A), ''), Chr($0D), ' ' ), ',', ';'), '<br>',' ');
   if cropHint and (length(desc) > cropLength) then
     SetLength(desc, cropLength);

   Result := desc;  
end;

function ExportPoint: string;
var
  s: string;
  hint : string;
  desc : string;
  attachment: string;
  n : integer;
  elev : string;
  lengthName : integer; //lenth of name (BoB)

begin
  s := IntToStr(Counter) + ',';
  Counter := Counter + 1;
  s := s + GC.ID + ' ';
  hint := '';
  attachment := '';
  elev := '-777';
  lengthName := 0  //lenth of name (BoB)

  {Prepare the hint}
  if Pos('WM',GC.ID) <> 1 then begin
    if showHint then
      hint := FixDesc(GC.Hint, cropHint, cropLength);

    if revHint then
      hint := Reverse(hint);
  end
  else begin	//PRO Waymarky
    if showHint then
	  hint := FixDesc(GC.CacheType, cropHint, cropLength);
  end;

  {Expecting copy of 'offline' folder in OziExplorer's data directory}
  if useOffline and (length(customURL) > 0) then
  begin
    attachment := ',,,,';
    attachment := attachment + customURL + '/cache/' + GC.ID[length(GC.ID)] + 
                                                          '/' + GC.ID + '.htm' + ',,';
    if forOZICE1 then
      attachment := ReplaceString(attachment, '\', '/')
    else
      attachment := ReplaceString(attachment, '/', '\');
  end;

  {Elevation information from tag}
  if GC.tagHavecategory(tagelev) then
    elev := m2feet(GC.TagValues(tagelev)); 

  s := s + GC.IDTag + ' ' + ReplaceString(UtfToAscii(GC.Name), ',', ';') + ',';
  lengthName := length(s)-1 //lenth of name BoB
  if length(hint)+lengthName>cropLength then  hint := Copy(hint,1,cropLength-lengthName); //Cutting long  hint  (BoB)
  s := s + GC.Lat + ',' + GC.Lon + ',';
  s := s + ReplaceString(FloatToStr(GC.Hidden), ',', '.');
  s := s +',0,1,3,' + ColorPoint + ',' + hint + ',0,0,' + intToStr(proximity) + ','+ elev +',6,0,17' + attachment;
  Result := s + CRLF;
  {Waypointy}
  if Pos('WM',GC.ID) <> 1 then begin
    for n := 0 to GC.Waypoints.Count - 1 do
    begin
      if (GC.IsFound and (not foundWptOn)) then continue;	//Ignore waypoints when is cache founded and option is set for not show!! 
      if GC.Waypoints[n].IsEmptyCoord  or (not GC.Waypoints[n].isListed) then 
        continue;
      s := IntToStr(Counter) + ',';
      Counter := Counter + 1;
      s := s + GC.ID + ' ' + GC.IDTag + ' ' + FixDesc(GC.Waypoints[n].FullName, false, 0) + ',';
      lengthName := length(s)-1 //lenth of name BoB
      s := s + GC.Waypoints[n].Lat + ',' + GC.Waypoints[n].Lon + ',';
      s := s + ReplaceString(FloatToStr(GC.Hidden), ',', '.');
      s := s + ',0,1,3,' + ColorWaypoint(GC.Waypoints[n]) + ',';

      if GC.Waypoints[n].IsFinal then 
        desc := FixDesc(hint + ':' + GC.Waypoints[n].Comment, cropHint, cropLength)
      else 
        desc := FixDesc(GC.Waypoints[n].Description, cropHint, cropLength); 
      if length(desc)+lengthName>cropLength then  desc := Copy(desc,1,cropLength-lengthName); //Cutting long  hint  (BoB)
      s := s + desc;
      s := s + ',0,0,' + intToStr(proximity) + ',-777,6,0,17' + attachment;
      Result := result + s + CRLF;
    end;
  end;
end;
